<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       optimum7.com
 * @since      1.0.0
 *
 * @package    Ffl_Api
 * @subpackage Ffl_Api/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Ffl_Api
 * @subpackage Ffl_Api/admin
 * @author     Serhat Durum <serhat@optimum7.com>
 */
class Ffl_Api_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     * @since    1.0.0
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Ffl_Api_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Ffl_Api_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/ffl-api-admin.css', array(), $this->version, 'all');

    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Ffl_Api_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Ffl_Api_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script('iris', admin_url('js/iris.min.js'), array('jquery-ui-draggable', 'jquery-ui-slider', 'jquery-touch-punch'), false, 1);
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/ffl-api-admin.js', array('jquery'), $this->version, true);
        wp_enqueue_style('forms');

    }

    public function ffl_load_menu()
    {
        add_menu_page('FFL API Settings Page', 'FFL API', 'manage_options', 'ffl-api-settings', array($this, 'ffl_api_settings_page'), '
dashicons-location-alt', 70);
        add_action('admin_init', array($this, 'register_ffl_api_settings'));
        add_action('woocommerce_product_options_general_product_data', array($this, 'ffl_option_group'));
        add_action('woocommerce_process_product_meta', array($this, 'ffl_save_fields'), 10, 2);

    }

    function ffl_save_fields($post_id)
    {

        $woocommerce_checkbox = isset($_POST['_firearm_product']) ? 'yes' : 'no';
        update_post_meta($post_id, '_firearm_product', $woocommerce_checkbox);

    }

    function ffl_option_group()
    {
        echo '<div class="options_group">';

        woocommerce_wp_checkbox(array(
            'id' => '_firearm_product',
            'value' => get_post_meta(get_the_ID(), '_firearm_product', true),
            'label' => 'This is a firearm product',
            'desc_tip' => true,
            'description' => 'Define this product as firearm by clicking the checkbox',
        ));

        echo '</div>';
    }

    function register_ffl_api_settings()
    {
        //register our settings
        register_setting('ffl-api-settings', 'ffl_api_key_option');
        register_setting('ffl-api-settings', 'ffl_api_gmaps_option');
        register_setting('ffl-api-settings', 'ffl_api_warning_message');
        register_setting('ffl-api-settings', 'ffl_list_background_option');
        register_setting('ffl-api-settings', 'ffl_list_text_option');
        register_setting('ffl-api-settings', 'ffl_init_map_location');
        register_setting('ffl-api-settings', 'ffl_map_show_modal', ['default' => 'yes']);
    }

    function ffl_api_settings_page()
    {

        ?>
        <div class="wrap">
            <a href="https://fflapi.com" target="_blank" style="display: inline-block">
                <img src="<?php echo plugin_dir_url(__FILE__) . 'images/ffl-logo.png' ?>" style="width: 150px">
            </a>
            <div class="postbox" style="padding: 10px;margin-top: 10px">

                <form method="post" action="options.php">

                    <?php settings_fields('ffl-api-settings'); ?>

                    <table class="form-table">

                        <tr valign="top">
                            <th scope="row">FFL API Key:</th>
                            <td>
                                <div class="user-pass-wrap">
                                    <div class="wp-pwd">
                                        <input type="password" style="width: 30%" name="ffl_api_key_option"
                                               aria-describedby="login_error" class="input password-input" size="20"
                                               value="<?php echo esc_attr(get_option('ffl_api_key_option')); ?>"/>
                                    </div>
                                </div>
                            </td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Google Maps API Key:</th>
                            <td><input type="text" style="width: 30%" name="ffl_api_gmaps_option"
                                       value="<?php echo esc_attr(get_option('ffl_api_gmaps_option')); ?>"/>
                            </td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Warning Message Modal</th>

                            <td>
                                <select name="ffl_map_show_modal">
                                    <?php
                                    $selects = Array(
                                        'yes' => 'Show',
                                        'no' => 'Hide'
                                    );
                                    foreach ($selects as $val => $key) {
                                        echo '<option value="' . $val . '" ';
                                        if (get_option('ffl_map_show_modal') == $val) {
                                            echo 'selected="selected" ';
                                        }
                                        echo '> ' . $key . '</option>';
                                    }
                                    ?>
                                </select>
                                <p id="warnHelp" class="form-text text-muted"></p>
                            </td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Firearm Warning Message:</th>
                            <td>
                                <textarea rows="5" cols="50"
                                          name="ffl_api_warning_message"><?php echo esc_attr(get_option('ffl_api_warning_message') != '' ? get_option('ffl_api_warning_message') : 'You have chosen your address as a shipping address for a firearm product. Unfortunately, we are not able to ship the firearms directly to your address. Please select the closest FFL from the map using your zip code.'); ?></textarea>
                                <p id="warnHelp" class="form-text text-muted">You can edit this message that appears on
                                    the checkout page</p></td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Background List Color:</th>
                            <td>
                                <input type="text" style="width: 30%" id="ffl_list_background_option"
                                       name="ffl_list_background_option"
                                       value="<?php echo esc_attr(get_option('ffl_list_background_option')); ?>"/>
                                <p id="warnHelp" class="form-text text-muted"></p>
                            </td>

                        </tr>
                        <tr valign="top">
                            <th scope="row">Background Selected Text Color:</th>
                            <td>
                                <input type="text" style="width: 30%" id="ffl_list_text_option"
                                       name="ffl_list_text_option"
                                       value="<?php echo esc_attr(get_option('ffl_list_text_option')); ?>"/>
                                <p id="warnHelp" class="form-text text-muted"></p>
                            </td>
                        </tr>

                        <tr valign="top">
                            <th scope="row">Location on Form:</th>

                            <td>
                                <select name="ffl_init_map_location">
                                    <?php
                                    $selects = Array(
                                        'woocommerce_checkout_order_review' => 'Order Review',
                                        'woocommerce_before_checkout_billing_form' => 'Above Billing Form',
                                        'woocommerce_after_checkout_billing_form' => 'Below Billing Form',
                                        'woocommerce_before_checkout_shipping_form' => 'Above Shipping Form',
                                        'woocommerce_after_checkout_shipping_form' => 'Below Shipping Form',
                                    );
                                    foreach ($selects as $val => $key) {
                                        echo '<option value="' . $val . '" ';
                                        if (get_option('ffl_init_map_location') == $val) {
                                            echo 'selected="selected" ';
                                        }
                                        echo '> ' . $key . '</option>';
                                    }
                                    ?>
                                </select>
                                <p id="warnHelp" class="form-text text-muted"></p>
                            </td>
                        </tr>

<!--                        <tr valign="top">-->
<!--                            <th scope="row">Select Theme:</th>-->
<!---->
<!--                            <td>-->
<!--                                <select style="width:11.3%;" name="ffl_init_theme">-->
<!--                                    --><?php
//                                    $selects = Array(
//                                        'ffl_light_theme' => 'Light (Default)',
//                                        'ffl_dark_theme' => 'Dark',
//                                    );
//                                    foreach ($selects as $val => $key) {
//                                        echo '<option value="' . $val . '" ';
//                                        if (get_option('ffl_init_map_location') == $val) {
//                                            echo 'selected="selected" ';
//                                        }
//                                        echo '> ' . $key . '</option>';
//                                    }
//                                    ?>
<!--                                </select>-->
<!--                                <p id="warnHelp" class="form-text text-muted"></p>-->
<!--                            </td>-->
<!--                        </tr>-->


                    </table>

                    <?php submit_button(); ?>
            </div>

            <div class="postbox" style="padding: 10px;margin-top: 10px">
                <h4 style="color: red">You just need to enable the checkbox from your woocommerce product's general
                    settings if your product is a firearm products.</h4>
                <p>Example:</p>
                <img src="<?php echo plugin_dir_url(__FILE__) . 'images/example.png' ?>">
            </div>


            </form>
        </div>
    <?php }


}
